mod components;

use std::env;
use sycamore::prelude::*;
use sycamore::futures::spawn_local_scoped;
use serde::Deserialize;
use log::{info, warn, error, Level, debug};

fn main() {
    console_log::init_with_level(Level::Debug).expect("Something went horribly wrong while setting the Logging Level");

    sycamore::render(|| view! {
        div(class="flex flex-col h-full") {
            div(class="min-w-full p-1 debug") {
                h1 { "Navbar" }
            }
            div(class="flex-1 grid grid-cols-5 debug") {
                div(class="col-span-1 debug") {
                    h1 { "Left Sidebar" }
                }
                div(class="col-span-3 debug") {
                    h1 { "Center Module" }
                }
                div(class="col-span-1 flex flex-col gap-1 p-2 debug") {
                    div(class="flex flex-row w-full gap-2 items-center") {
                        h1(class="flex flex-1") { "Current Queue" }
                        p { "Fetch" }
                        p { "Upload" }
                    }
                    components::separator(width = 100)
                    crate::render_playlist {}
                }
            }
            div(class="min-h-24 h-24 debug") {
                h3 { "Player Module" }
            }
        }
    });
}

#[derive(Deserialize)]
struct PlayQueueApi {
    #[serde(alias="subsonic-response")]
    subsonic_response: SubsonicResponse
}

#[derive(Deserialize)]
struct SubsonicResponse {
    status: String,
    version: String,
    r#type: String,
    #[serde(alias="serverVersion")]
    server_version: String,
    #[serde(alias="openSubsonic")]
    open_subsonic: bool,
    #[serde(alias="playQueue")]
    play_queue: PlayQueue,
}

#[derive(Debug, Deserialize)]
struct PlayQueue {
    entry: Vec<Song>,
    current: Option<String>,
    position: Option<usize>,
    username: String,
    changed: String,
    #[serde(alias="changedBy")]
    changed_by: String
}

#[derive(Clone, Debug, Deserialize)]
struct Song {
    id: String,
    parent: String,
    #[serde(alias="isDir")]
    is_dir: bool,
    title: String,
    album: String,
    artist: String,
    track: usize,
    year: usize,
    genre: String,
    genres: Vec<Genre>,
    #[serde(alias="coverArt")]
    cover_art: String,
    size: usize,
    #[serde(alias="contentType")]
    content_type: String,
    suffix: String,
    duration: usize,
    #[serde(alias="bitRate")]
    bit_rate: usize,
    path: String,
    #[serde(alias="playCount")]
    play_count: usize,
    played: String,
    #[serde(alias="discNumber")]
    disc_number: usize,
    created: String,
    #[serde(alias="albumId")]
    album_id: String,
    #[serde(alias="artistId")]
    artist_id: String,
    r#type: String,
    #[serde(alias="userRating")]
    user_rating: Option<usize>,
    #[serde(alias="isVideo")]
    is_video: bool,
    bpm: usize,
    comment: String
}

#[derive(Debug, Clone, Deserialize)]
struct Genre {
    name: String
}

async fn test_api() -> Vec<Song> {
    const API_BASE: &str = "https://music.neshweb.net/rest";


    let salt = "testsalt1234"; // TODO: random salt please

    let salted_password = format!("{password}{salt}");

    let hash = md5::compute(salted_password.as_bytes());

    let res = match reqwest::get(format!("{API_BASE}/getPlayQueue.view?u={user}&t={:x}&s={salt}&v=1.16.1&c={}&f=json", hash, env!("CARGO_PKG_NAME"))).await {
        Ok(data) => data,
        Err(e) => {
            error!("Error: {e}");
            return vec![]
        },
    };

    debug!("Still alive!");

    let data = match res.json::<PlayQueueApi>().await {
        Ok(data) => data,
        Err(e) => {
            error!("Error: {e}");
            return vec![]
        },
    };



    data.subsonic_response.play_queue.entry.iter().map(|elem| {
        elem.clone()
    }).collect()

    /*
    let res = reqwest::blocking::get(format!("{api_base}/getPlayQueue.view?u={user}&t={:x}&s={salt}&v=1.16.1&c=myapp&f=json", hash)).unwrap();

    let text = res.text().unwrap();

    println!("{text}");

    let song_id = "81983bfae2fdddfa5d3dc181d61d987c";
    let song_pos = 1;

    let res_save = reqwest::blocking::get(format!("{api_base}/savePlayQueue.view?u={user}&t={:x}&s={salt}&v=1.16.1&c={}&f=json&id={song_id}&pos={song_pos}", hash, env!("CARGO_PKG_NAME"))).unwrap();

    let text_save = res_save.text().unwrap();

    println!("{text_save}");*/
}

#[component]
fn render_playlist<G: Html>() -> View<G> {
    let list: Signal<Vec<Song>> = create_signal(vec![]);

    info!("Info Test");
    debug!("Debug Test");

    spawn_local_scoped(async move {
        debug!("Moving into spawned task");
        let res = test_api().await;
        debug!("Data from spawned task returned");
        list.set(res);
    });

    create_effect(move || {
        debug!("List is: {:#?}", list.get_clone());
    });

    view! {
        (View::new_fragment(list.get_clone().iter().map(|elem: &Song| {
            let song = elem.clone();
            view! {
                p { (song.artist) " - " (song.title) " (" (seconds_to_duration(song.duration)) ")" }
            }
        }).collect()))
    }
}

fn seconds_to_duration(raw_seconds: usize) -> String {
    let minutes = raw_seconds / 60;
    let seconds = raw_seconds % 60;
    format!("{minutes}:{seconds}")
}
